@@ -20,13 +20,13 @@ LONBGLSS <- function(y,e,X,g,w,z,k,max.steps,sparse, structure){
   hatLambdaSqStar1=1
   hatLambdaSqStar2=1
   hatSigmaSq=1
-  hatPhiSq=1
+  hatPhi1Sq= hatPhi2Sq=1
   a0=aStar=1
   b0=bStar=1
   alpha=1
   gamma=1
-  alpha1=1
-  gamma1=1
+  alpha1=0
+  gamma1=0
   mu0=mu1=1
   nu0=nu1=1
   debugging=FALSE
@@ -36,16 +36,16 @@ LONBGLSS <- function(y,e,X,g,w,z,k,max.steps,sparse, structure){
   if(sparse){
     fit=switch (structure,
                 "bi-level" = BGLSS(y,E,g,w,z,q,o,k,max.steps,hatAlpha,hatBeta,hatAta,hatEta,invSigAlpha0,hatInvTauSq1,hatInvTauSq22,hatPiBeta,hatPiEta,hatLambdaSqStar1
-                                ,hatLambdaSqStar2,hatSigmaSq,hatPhiSq,a0,b0,aStar,bStar,alpha,gamma,alpha1,gamma1,mu0,mu1,nu0,nu1,progress),
+                                ,hatLambdaSqStar2,hatSigmaSq,hatPhi1Sq,hatPhi2Sq,a0,b0,aStar,bStar,alpha,gamma,alpha1,gamma1,mu0,mu1,nu0,nu1,progress),
                 "individual" = BLSS(y,E,g,w,z,q,k,max.steps,hatAlpha,hatBeta,hatAta,hatEta,invSigAlpha0,hatInvTauSq1,hatInvTauSq21,hatPiBeta,hatPiEta,hatLambdaSqStar1
-                                    ,hatLambdaSqStar2,hatSigmaSq,hatPhiSq,a0,b0,aStar,bStar,alpha,gamma,alpha1,gamma1,mu0,mu1,nu0,nu1,progress)
+                                    ,hatLambdaSqStar2,hatSigmaSq,hatPhi1Sq,hatPhi2Sq,a0,b0,aStar,bStar,alpha,gamma,alpha1,gamma1,mu0,mu1,nu0,nu1,progress)
     )
   }else{
     fit=switch (structure,
                 "bi-level" = BGL(y,E,g,w,q,o,k,max.steps,hatAlpha,hatBeta,hatEta,hatAta,z,invSigAlpha0,hatInvTauSq1,hatInvTauSq22,hatLambdaSqStar1
-                              ,hatLambdaSqStar2,hatSigmaSq,a0,b0,aStar,bStar,hatPhiSq,alpha,gamma,alpha1,gamma1,progress),
+                              ,hatLambdaSqStar2,hatSigmaSq,a0,b0,aStar,bStar,hatPhi1Sq,hatPhi2Sq,alpha,gamma,alpha1,gamma1,progress),
                 "individual" = BL(y,E,g,w,q,k,max.steps,hatAlpha,hatBeta,hatEta,hatAta,z,invSigAlpha0,hatInvTauSq1,hatInvTauSq21,hatLambdaSqStar1
-                                  ,hatLambdaSqStar2,hatSigmaSq,a0,b0,aStar,bStar,hatPhiSq,alpha,gamma,alpha1,gamma1,progress)
+                                  ,hatLambdaSqStar2,hatSigmaSq,a0,b0,aStar,bStar,hatPhi1Sq,hatPhi2Sq,alpha,gamma,alpha1,gamma1,progress)
     )
   }
   out = list(GS.gamma1 = fit$GS.alpha[,1:(q-o)], GS.gamma0 = fit$GS.alpha[,-(1:(q-o))],GS.gamma2 = fit$GS.beta,
