@@ -11,7 +11,7 @@ using namespace arma;
 
 
 // [[Rcpp::export()]]
-Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat z, unsigned int q,unsigned int k, int maxSteps, arma::vec hatAlpha, arma::vec hatBeta, arma::mat hatAta, arma::vec hatEta, arma::vec hatInvSigM0, arma::vec hatInvTauSq0, arma::vec hatInvTauSqStar, double hatPi0, double hatPiStar,double hatLambdaSq0, double hatLambdaSqStar, double hatSigmaSq, double hatPhiSq, double a0, double b0, double aStar, double bStar, double alpha, double gamma, double alpha1,double gamma1, double mu0, double muStar, double nu0, double nuStar, int progress)
+Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat z, unsigned int q,unsigned int k, int maxSteps, arma::vec hatAlpha, arma::vec hatBeta, arma::mat hatAta, arma::vec hatEta, arma::vec hatInvSigM0, arma::vec hatInvTauSq0, arma::vec hatInvTauSqStar, double hatPi0, double hatPiStar,double hatLambdaSq0, double hatLambdaSqStar, double hatSigmaSq, double Phi1Sq, double Phi2Sq, double a0, double b0, double aStar, double bStar, double alpha, double gamma, double alpha1,double gamma1, double mu0, double muStar, double nu0, double nuStar, int progress)
 {
   unsigned int n = g.n_rows, m = g.n_cols, c = z.n_cols,p = w.n_cols, n1 = n/k;
   arma::mat gsAlpha(maxSteps, q),
@@ -27,7 +27,8 @@ Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat
   gsLambdaStar(maxSteps),
   gsSigmaSq(maxSteps),
   gsPi0(maxSteps),
-  gsPhiSq(maxSteps),
+  gsPhi1Sq(maxSteps),
+  gsPhi2Sq(maxSteps),
   gsPiStar(maxSteps)
 
     ;
@@ -43,34 +44,50 @@ Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat
 
   for (int t = 0; t < maxSteps; t++) {
     // m|y, r0, r.star
-    varM = arma::inv(tBmBm/hatSigmaSq + invSigM0);
-    res = y - (g * hatBeta + w * hatEta);
+    arma::mat Zblock(n, c*n1, arma::fill::zeros);
     for(unsigned int i=0;i<n1;i++){
-      res.subvec((i*k), (i*k+k-1)) -= z*hatAta.col(i);
+      Zblock.submat(i*k,c*i,i*k + k - 1, c*i + 1) = z;
     }
+    varM = arma::inv(tBmBm/hatSigmaSq + invSigM0);
+    res = y - (g * hatBeta + w * hatEta+ Zblock * arma::vectorise(hatAta));
     meanM = varM * (e.t() * res/hatSigmaSq);
     hatAlpha = mvrnormCpp(meanM, varM);
     res -= e * hatAlpha;
     gsAlpha.row(t) = hatAlpha.t();
 
     // ata|
+    res+= Zblock*arma::vectorise(hatAta);
+
+    arma::vec z0 = z.col(0);  // k x 1
+    arma::vec z1 = z.col(1);  // k x 1
+
+    for (unsigned int i = 0; i < n1; i++) {
+
+      const unsigned int s = i * k;
+      const unsigned int e = s + k - 1;
+
+      arma::vec r = res.subvec(s, e);
+
+      // ata0
+      double t00 = arma::dot(z0, z0) / hatSigmaSq;
+      double b0  = arma::dot(z0, r)  / hatSigmaSq;
+      double var0  = 1.0 / (t00 + 1.0 / Phi1Sq);
+      double mean0 = var0 * b0;
+      hatAta(0, i) = R::rnorm(mean0, std::sqrt(var0));
+
+      // ata1
+      double t11 = arma::dot(z1, z1) / hatSigmaSq;
+      double b1  = arma::dot(z1, r)  / hatSigmaSq;
+      double var1  = 1.0 / (t11 + 1.0 / Phi2Sq);
+      double mean1 = var1 * b1;
+      hatAta(1, i) = R::rnorm(mean1, std::sqrt(var1));
 
-    for(unsigned int i=0;i<n1;i++){
-      res.subvec((i*k), (i*k+k-1)) += z * hatAta.col(i);
-      arma::mat tzz =  z.t()*z;
-      arma::mat A = tzz/hatSigmaSq;
-      arma::vec B = z.t()*res.subvec((i*k), (i*k+k-1))/hatSigmaSq;
-      arma:: mat T(c,c);
-      T = T.eye();
-      arma::mat invhatPhiSq = 1/hatPhiSq*T;
-      arma::mat varAta = arma::inv(A+invhatPhiSq);
-      arma::vec meanAta = varAta*B;
-      hatAta.col(i) = mvrnormCpp(meanAta, varAta);
-      res.subvec((i*k), (i*k+k-1)) -= z * hatAta.col(i);
     }
 
     gsAta.row(t) = arma::vectorise(hatAta).t();
 
+    res-= Zblock*arma::vectorise(hatAta);
+
     for(unsigned int j=0; j<m; j++){
       temp0 = 1/(tB0B0Diag(j) + hatInvTauSq0(j));
       varR0 = hatSigmaSq * temp0;
@@ -160,18 +177,21 @@ Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat
     hatSigmaSq = 1/R::rgamma(shapeSig, 1/rateSig);
     gsSigmaSq(t) = hatSigmaSq;
 
-    //phi;
-    double shapePhi, ratePhi;
-    shapePhi = alpha1 + n1*c/2;
-    double diff;
-    diff=0;
-    for(unsigned int i=0;i<n1;i++){
-      diff= diff+0.5*(arma::accu(square(hatAta.col(i))));
-    }
+    //phi1sq, phi2sq;
+    double diff1 = 0.5 * arma::accu( arma::square(hatAta.row(0)) );
+    double diff2 = 0.5 * arma::accu( arma::square(hatAta.row(1)) );
+
+    double shape1 = alpha1 + n1 / 2.0;
+    double rate1  = gamma1 + diff1;
+    Phi1Sq = 1.0 / R::rgamma(shape1, 1.0 / rate1);
+
+    double shape11 = alpha1 + n1 / 2.0;
+    double rate11  = gamma1 + diff2;
+    Phi2Sq = 1.0 / R::rgamma(shape11, 1.0 / rate11);
+
+    gsPhi1Sq(t) = Phi1Sq;
+    gsPhi2Sq(t) = Phi2Sq;
 
-    ratePhi = gamma1 + diff;
-    hatPhiSq = 1/R::rgamma(shapePhi, 1/ratePhi);
-    gsPhiSq(t) = hatPhiSq;
 
     // pi.0|
     double shape1_0 = mu0 + arma::accu(hatBeta == 0);
@@ -201,21 +221,7 @@ Rcpp::List BLSS(arma::vec y, arma::mat e, arma::mat g, arma:: mat w, arma:: mat
                             Rcpp::Named("GS.beta") = gsBeta,
                             Rcpp::Named("GS.ata") = gsAta,
 
-                            Rcpp::Named("GS.eta") = gsEta,
-
-                            Rcpp::Named("GS.invTAUsq.0") = gsInvTauSq0,
-
-                            Rcpp::Named("GS.invTAUsq.star") = gsInvTauSqStar,
-                            Rcpp::Named("GS.pi0") = gsPi0,
-
-                            Rcpp::Named("GS.pi.star") = gsPiStar,
-                            Rcpp::Named("GS.lambda.sq.0") = gsLambda0,
-
-                            Rcpp::Named("GS.lambda.sq.star") = gsLambdaStar,
-                            Rcpp::Named("GS.sigma.sq") = gsSigmaSq,
-                            Rcpp::Named("GS.l0") = gsL0,
-
-                            Rcpp::Named("GS.lS") = gsLS
+                            Rcpp::Named("GS.eta") = gsEta
 
   );
 
